﻿local options = {
    type = 'group',
    args = {
        lock = {
            type = "toggle",
            name = "Lock Sphere and Buttons",
            desc = "Lock the main sphere and buttons so they can't be moved.",
            get = function() return Cryolysis2.db.profile.LockSphere end,
            set = function(v) Cryolysis2.db.profile.LockSphere = v end,
            order = 1
        },
        constrict = {
            type = "toggle",
            name = "Constrict Buttons to Sphere",
            desc = "Lock the buttons in place around the main sphere.",
            get = function() return Cryolysis2.db.profile.LockButtons end,
            set = function(v) 
                Cryolysis2.db.profile.LockButtons = v 
                Cryolysis2:UpdateAllButtonPositions()
                end,
            order = 2
        },
        sm = {
            type = "toggle",
            name = "Short Messages",
            desc = "Display shortened versions of polymorph, teleport, portal, and mount messages.",
            get = function() return Cryolysis2.db.profile.SM end,
            set = function(v) Cryolysis2.db.profile.SM = v end,
            order = 2.4
        },
        poly = {
            type = "toggle",
            name = "Polymorph Messages",
            desc = "Display a message in chat when casting polymorph.",
            get = function() return Cryolysis2.db.profile.PolyMessage end,
            set = function(v) Cryolysis2.db.profile.PolyMessage = v end,
            order = 2.7
        },
        blank1 = {
            type = "header",
            name = " ",
            order = 3
        },
        timersettings = {
            type = "header",
            name = "Timer Settings",
            order = 15
        },
        timertoggle = {
            type = "toggle",
            name = "Timers On/Off",
            desc = "Turn timers on or off.",
            get = function() return Cryolysis2.db.profile.ShowSpellTimers end,
            set = function(v) Cryolysis2.db.profile.ShowSpellTimers = v end,
            order = 16
        },
        cooldown = {
            type = "group",
            name = "Cooldown Timers",
            desc = "Select which spells have their cooldowns shown in the timer window.",
            order = 17,
            args = {
            
            }
        },
        mountmsg = {
            type = "group",
            name = "Debuff Timers",
            desc = "Select which spells have their debuffs tracked in the timer window.",
            order = 18,
            args = {
            
            }
        },
        blank4 = {
            type = "header",
            name = " ",
            order = 19
        },
        graphicalsettings = {
            type = "header",
            name = "Button Settings",
            order = 20
        },
        msbutton = {
            type = "group",
            name = "Main Sphere",
            desc = "Adjust various settings for the main sphere.",
            order = 20.5,
            args = {
                mshide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the main sphere.",
                    get = function() return Cryolysis2.db.profile.MSHide end,
                    set = function(v)
                        Cryolysis2.db.profile.MSHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                insphere = {
                    type = "range",
                    name = "Text",
                    desc = "Adjust what information is displayed on the sphere. 1 = Nothing, 2 = Food/Drink, 3 = Drink/Food, 4 = Current Health, 5 = Health %, 6 = Current Mana, 7 = Mana %, 8 = Mana Gem Cooldown, 9 = Evocation Cooldown",
                    get = function() return Cryolysis2.db.profile.InnerSphere end,
                    set = function(v)
                        Cryolysis2.db.profile.InnerSphere = v 
                        Cryolysis2:UpdateInnerSphere() 
                    end,
                    min = 1,
                    max = 9,
                    step = 1,
                    order = 1.5
                },
                spheremb = {
                    type = "range",
                    name = "Inner Sphere",
                    desc = "Adjust what clicking the inner sphere does. 1 = Nothing, 2 = Eat and drink, 3 = Evocation, 4 = Polymorph, 5 = Use Mana Gem, 6 = Blink",
                    get = function() return Cryolysis2.db.profile.MainButton end,
                    set = function(v) 
                        Cryolysis2.db.profile.MainButton = v
                        Cryolysis2:UpdateSphereAttributes()
                        end,
                    min = 1,
                    max = 6,
                    step = 1,
                    order = 2
                },
                outsphere = {
                    type = "range",
                    name = "Outer Sphere",
                    desc = "Adjust what information is displayed using the outer sphere. 1 = Nothing, 2 = Health, 3 = Mana, 4 = Mana Gem Cooldown, 5 = Evocation Cooldown",
                    get = function() return Cryolysis2.db.profile.OuterSphere end,
                    set = function(v) 
                        Cryolysis2.db.profile.OuterSphere = v 
                        Cryolysis2:UpdateOuterSphere()
                        end,
                    min = 1,
                    max = 5,
                    step = 1,
                    order = 3
                },
                angleadj = {
                    type = "range",
                    name = "Angle",
                    desc = "Adjust the position of the buttons around the sphere.",
                    get = function() return Cryolysis2.db.profile.Angle end,
                    set = function(v) 
                        Cryolysis2.db.profile.Angle = v
                        Cryolysis2:UpdateAllButtonPositions()
                        end,
                    min = 0,
                    max = 360,
                    step = 18,
                    order = 4
                },
                scalems = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the main sphere.",
                    get = function() return Cryolysis2.db.profile.MainSphereScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.MainSphereScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true,
                    order = 5
                }
            }
        },
        foodbutton = {
            type = "group",
            name = "Food Button",
            desc = "Adjust various settings for the food button.",
            order = 21,
            args = {
                foodhide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.FoodHide end,
                    set = function(v)
                        Cryolysis2.db.profile.FoodHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                foodcount = {
                    type = "toggle",
                    name = "Show Food Count",
                    desc = "Show how much conjured food you have on the food button.",
                    get = function() return Cryolysis2.db.profile.ShowFoodCountText end,
                    set = function(v) Cryolysis2.db.profile.ShowFoodCountText = v end
                },
                scalefood = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the food button.",
                    get = function() return Cryolysis2.db.profile.FoodButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.FoodButtonScale = v
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        waterbutton = {
            type = "group",
            name = "Water Button",
            desc = "Adjust various settings for the water button.",
            order = 22,
            args = {
                waterhide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.WaterHide end,
                    set = function(v)
                        Cryolysis2.db.profile.WaterHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                watercount = {
                    type = "toggle",
                    name = "Show Water Count",
                    desc = "Show how much conjured water you have on the water button.",
                    get = function() return Cryolysis2.db.profile.ShowWaterCountText end,
                    set = function(v) Cryolysis2.db.profile.ShowWaterCountText = v end
                },
                scalewater = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the water button.",
                    get = function() return Cryolysis2.db.profile.WaterButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.WaterButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        gembutton = {
            type = "group",
            name = "Mana Gem Button",
            desc = "Adjust various settings for the mana gem button.",
            order = 23,
            args = {
                gemhide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.GemHide end,
                    set = function(v)
                        Cryolysis2.db.profile.GemHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                gem = {
                    type = "toggle",
                    name = "Show Cooldown",
                    desc = "Show the mana gem cooldown on the mana gem button.",
                    get = function() return Cryolysis2.db.profile.ShowManaGemCooldownText end,
                    set = function(v) 
                        Cryolysis2.db.profile.ShowManaGemCooldownText = v
                        Cryolysis2:Cryo2_UpdateManaGemCooldown() 
                        end,
                    order = 7
                },
                scalegem = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the mana gem button.",
                    get = function() return Cryolysis2.db.profile.ManaGemButtonScale end,
                    set = function(v)
                        Cryolysis2.db.profile.ManaGemButtonScale = v
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        leftcustom = {
            type = "group",
            name = "Left Custom Button",
            desc = "Adjust various settings for the left custom button.",
            order = 24,
            args = {
                lefthide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.LeftHide end,
                    set = function(v)
                        Cryolysis2.db.profile.LeftHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                scaleleft = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the left custom button.",
                    get = function() return Cryolysis2.db.profile.LeftCustomButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.LeftCustomButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                },
                leftleftclick = {
                    type = "text",
                    name = "Left Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.LeftSpellLeft end,
                    set = function(v) 
                        Cryolysis2.db.profile.LeftSpellLeft = v
                        Cryolysis2:UpdateLeftButton(1) 
                        end
                },
                leftrightclick = {
                    type = "text",
                    name = "Right Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.LeftSpellRight end,
                    set = function(v) 
                        Cryolysis2.db.profile.LeftSpellRight = v
                        Cryolysis2:UpdateLeftButton(2) 
                        end
                },
                leftmidclick = {
                    type = "text",
                    name = "Middle Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.LeftSpellMid end,
                    set = function(v) 
                        Cryolysis2.db.profile.LeftSpellMid = v 
                        Cryolysis2:UpdateLeftButton(3) 
                        end
                }
            }
        },
        evocationbutton = {
            type = "group",
            name = "Evocation Button",
            desc = "Adjust various settings for the evocation button.",
            order = 24.5,
            args = {
                evochide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.EvocHide end,
                    set = function(v)
                        Cryolysis2.db.profile.EvocHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                evoc = {
                    type = "toggle",
                    name = "Show Cooldown",
                    desc = "Show evocation's cooldown on the evocation button.",
                    get = function() return Cryolysis2.db.profile.ShowEvocationCooldownText end,
                    set = function(v) 
                        Cryolysis2.db.profile.ShowEvocationCooldownText = v
                        Cryolysis2:Cryo2_UpdateEvocationCooldown()
                        end,
                    order = 6
                },
                scaleevoc = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the evocation button.",
                    get = function() return Cryolysis2.db.profile.EvocationButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.EvocationButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        rightcustom = {
            type = "group",
            name = "Right Custom Button",
            desc = "Adjust various settings for the right custom button.",
            order = 25,
            args = {
                righthide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.RightHide end,
                    set = function(v)
                        Cryolysis2.db.profile.RightHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                scalerightcustom = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the right custom button.",
                    get = function() return Cryolysis2.db.profile.RightCustomButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.RightCustomButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                },
                rightleftclick = {
                    type = "text",
                    name = "Left Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.RightSpellLeft end,
                    set = function(v) 
                        Cryolysis2.db.profile.RightSpellLeft = v
                        Cryolysis2:UpdateRightButton(1) 
                        end
                },
                rightrightclick = {
                    type = "text",
                    name = "Right Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.RightSpellRight end,
                    set = function(v) 
                        Cryolysis2.db.profile.RightSpellRight = v
                        Cryolysis2:UpdateRightButton(2) 
                        end
                },
                rightmidclick = {
                    type = "text",
                    name = "Middle Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.RightSpellMid end,
                    set = function(v) 
                        Cryolysis2.db.profile.RightSpellMid = v 
                        Cryolysis2:UpdateRightButton(3) 
                        end
                }
            }
        },
        spellbutton = {
            type = "group",
            name = "Spell Menu Button",
            desc = "Adjust various settings for the spell menu button.",
            order = 25.5,
            args = {
                buffhide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.BuffHide end,
                    set = function(v)
                        Cryolysis2.db.profile.BuffHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                feathers = {
                    type = "toggle",
                    name = "Light Feather Count",
                    desc = "Show how many light feathers you have on the slow fall button.",
                    get = function() return Cryolysis2.db.profile.ShowFeatherCountText end,
                    set = function(v) Cryolysis2.db.profile.ShowFeatherCountText = v end
                },
                powder = {
                    type = "toggle",
                    name = "Arcane Powder Count",
                    desc = "Show how much arcane powder you have on the arcane intellect button.",
                    get = function() return Cryolysis2.db.profile.ShowPowderCountText end,
                    set = function(v) Cryolysis2.db.profile.ShowPowderCountText = v end
                },
                scalespell = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the spell menu button.",
                    get = function() return Cryolysis2.db.profile.SpellButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.SpellButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        mountbutton = {
            type = "group",
            name = "Mount Button",
            desc = "Adjust various settings for the mount button.",
            order = 26,
            args = {
                mounthide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.MountHide end,
                    set = function(v)
                        Cryolysis2.db.profile.MountHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                mountmsg = {
                    type = "toggle",
                    name = "Messages",
                    desc = "Display a message in chat when getting on your mount.",
                    get = function() return Cryolysis2.db.profile.MountMessage end,
                    set = function(v) Cryolysis2.db.profile.MountMessage = v end,
                    order = 13
                },
                scalemount = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the mount button.",
                    get = function() return Cryolysis2.db.profile.MountButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.MountButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        teleandportbutton = {
            type = "group",
            name = "Teleport/Portal Menu Button",
            desc = "Adjust various settings for the teleport/portal menu button.",
            order = 27,
            args = {
                porthide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.PortHide end,
                    set = function(v)
                        Cryolysis2.db.profile.PortHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                teleandport = {
                    type = "toggle",
                    name = "Messages",
                    desc = "Display a message in chat when teleporting or creating a portal.",
                    get = function() return Cryolysis2.db.profile.TeleAndPortMessage end,
                    set = function(v) Cryolysis2.db.profile.TeleAndPortMessage = v end,
                    order = 12
                },
                scaleteleandport = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the teleport/portal button.",
                    get = function() return Cryolysis2.db.profile.TeleAndPortButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.TeleAndPortButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                }
            }
        },
        bottomcustom = {
            type = "group",
            name = "Bottom Custom Button",
            desc = "Adjust various settings for the bottom custom button.",
            order = 28,
            args = {
                bottomhide = {
                    type = "toggle",
                    name = "Hide",
                    desc = "Show or hide the button.",
                    get = function() return Cryolysis2.db.profile.BottomHide end,
                    set = function(v)
                        Cryolysis2.db.profile.BottomHide = v
                        Cryolysis2:UpdateHideShow()
                        end,
                    order = 1
                },
                scalebottomcustom = {
                    type = "range",
                    name = "Scale",
                    desc = "Scale the size of the bottom custom button.",
                    get = function() return Cryolysis2.db.profile.RightCustomButtonScale end,
                    set = function(v) 
                        Cryolysis2.db.profile.BottomCustomButtonScale = v 
                        Cryolysis2:UpdateAllButtonPositions()
                        Cryolysis2:UpdateAllButtonSizes()
                        end,
                    min = .5,
                    max = 2,
                    isPercent = true
                },
                bottomleftclick = {
                    type = "text",
                    name = "Left Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.BottomSpellLeft end,
                    set = function(v) 
                        Cryolysis2.db.profile.BottomSpellLeft = v
                        Cryolysis2:UpdateBottomButton(1) 
                        end
                },
                bottomrightclick = {
                    type = "text",
                    name = "Right Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.BottomSpellRight end,
                    set = function(v) 
                        Cryolysis2.db.profile.BottomSpellRight = v
                        Cryolysis2:UpdateBottomButton(2) 
                        end
                },
                bottommidclick = {
                    type = "text",
                    name = "Middle Click Spell",
                    desc = "Type in the name of the spell that will be cast when left clicking this button.",
                    usage = "<any string>",
                    get = function() return Cryolysis2.db.profile.BottomSpellMid end,
                    set = function(v) 
                        Cryolysis2.db.profile.BottomSpellMid = v 
                        Cryolysis2:UpdateBottomButton(3) 
                        end
                }
            }
        }
    }
}

local GroundMounts = {
	-- [1] Frostwolf Howler Icon
	{ "Horn of the Frostwolf Howler" },
	-- [2] Ram Icon
	{ "Stormpike Battle Charger", "Black Ram", "Black War Ram", "Brown Ram", "Frost Ram", "Gray Ram", "Swift Brown Ram", "Swift Gray Ram", "Swift White Ram", "White Ram", },
	-- [3] Raptor Icon
	{ "Swift Razzashi Raptor", "Swift Blue Raptor", "Swift Olive Raptor", "Swift Orange Raptor", "Whistle of the Black War Raptor", "Whistle of the Emerald Raptor", "Whistle of the Ivory Raptor", "Whistle of the Mottled Red Raptor", "Whistle of the Turquoise Raptor", "Whistle of the Violet Raptor" },
	-- [4] Yellow Tiger Icon
	{ "Swift Zulian Tiger" },
	-- [5] Undead Horse Icon
	{ "Deathcharger's Reins", "Blue Skeletal Horse", "Blue Skeletal Warhorse", "Deathcharger's Reins", "Brown Skeletal Horse", "Green Skeletal Warhorse", "Purple Skeletal Warhorse", "Red Skeletal Horse", "Red Skeletal Warhorse" },
	-- [6] Mechanostrider Icon
	{ "Black Battlestrider", "Blue Mechanostrider", "Green Mechanostrider", "Icy Blue Mechanostrider Mod A", "Red Mechanostrider", "Swift Green Mechanostrider", "Swift White Mechanostrider", "Swift Yellow Mechanostrider", "Unpainted Mechanostrider", "White Mechanostrider Mod A" },
	-- [7] Brown Horse Icon
	{ "Black Stallion Bridle", "Brown Horse Bridle", "Chestnut Mare Bridle", "Palomino Bridle", "Pinto Bridle", "Swift Brown Steed", "Swift Palomino", "Swift White Steed", "White Stallion Bridle" },
	-- [8] Brown Kodo Icon
	{ "Black War Kodo", "Brown Kodo", "Great Brown Kodo" },
	-- [9] War Steed Icon
	{ "Black War Steed Bridle" },
	-- [10] Gray Kodo Icon
	{ "Gray Kodo", "Great Gray Kodo", "Great White Kodo" },
	-- [11] Green Kodo Icon
	{ "Green Kodo", "Teal Kodo" },
	-- [12] White Wolf Icon
	{ "Horn of the Arctic Wolf", "Horn of the Dire Wolf", "Horn of the Swift Gray Wolf", "Horn of the Swift Timber Wolf" },
	-- [13] Black Wolf Icon
	{ "Horn of the Black War Wolf", "Horn of the Brown Wolf", "Horn of the Red Wolf", "Horn of the Swift Brown Wolf", "Horn of the Timber Wolf" },
	-- [14] Black Tiger Icon
	{ "Reins of the Black War Tiger", "Reins of the Striped Nightsaber" },
	-- [15] White Tiger Icon
	{ "Reins of the Frostsaber", "Reins of the Nightsaber", "Reins of the Spotted Frostsaber", "Reins of the Striped Frostsaber", "Reins of the Swift Frostsaber", "Reins of the Swift Mistsaber", "Reins of the Swift Stormsaber" },
	-- [16] Red Tiger Icon
	{ "Reins of the Winterspring Frostsaber" },
	-- [17] Black Qiraji Resonating Crystal
	{ "Black Qiraji Resonating Crystal" },
    -- [18] Talbuk Icon
    { "Reins of the Cobalt Riding Talbuk", "Reins of the Cobalt War Talbuk", "Reins of the Silver War Talbuk", "Reins of the Silver Riding Talbuk", "Reins of the Tan Riding Talbuk", "Reins of the Tan War Talbuk", "Reins of the White Riding Talbuk", "Reins of the White War Talbuk" },
    -- [19] Brown Elekk Icon
    { "Brown Elekk" },
    -- [20] Gray Elekk Icon
    { "Gray Elekk" },
    -- [21] Purple Elekk Icon
    { "Purple Elekk" },
    -- [22] Great Blue Elekk Icon
    { "Great Blue Elekk" },
    -- [23] Great Green Elekk Icon
    { "Great Green Elekk" },
    -- [24] Great Purple Elekk Icon
    { "Great Purple Elekk" },
    -- [25] Hawkstrider Icon
    { "Black Hawkstrider" },
    -- [26] Hawkstrider Icon
    { "Blue Hawkstrider" },
    -- [27] Hawkstrider Icon
    { "Purple Hawkstrider" },
    -- [28] Hawkstrider Icon
    { "Red Hawkstrider" },
    -- [29] Hawkstrider Icon
    { "Swift Green Hawkstrider" },
    -- [30] Hawkstrider Icon
    { "Swift Pink Hawkstrider" },
    -- [31] Hawkstrider Icon
    { "Swift Purple Hawkstrider" },
    -- [32] Fiery Warhorse Icon
    { "Fiery Warhorse's Reins" },
}

local FlyingMounts = {

    -- [1] Gryphon Icon
    { "Ebon Gryphon", "Golden Gryphon", "Snowy Gryphon", "Swift Green Gryphon", "Swift Red Gryphon", "Swift Purple Gryphon", "Swift Blue Gryphon" },
    -- [2] Windrider Icon
    { "Blue Windrider", "Green Windrider", "Tawny Windrider", "Swift Green Windrider", "Swift Purple Windrider", "Swift Red Windrider", "Swift Yellow Windrider" },
    -- [3] Netherwing Drake Icon
    { "Reins of the Azure Netherwing Drake", "Reins of the Cobalt Netherwing Drake", "Reins of the Onyx Netherwing Drake", "Reins of the Purple Netherwing Drake", "Reins of the Veridian Netherwing Drake", "Reins of the Violet Netherwing Drake" },
    -- [4] Swift Nether Drake Icon
    { "Swift Nether Drake" },
    -- [5] Riding Nether Ray Icon
    { "Blue Riding Nether Ray", "Green Riding Nether Ray", "Purple Riding Nether Ray", "Red Riding Nether Ray", "Silver Riding Nether Ray" },
    -- [6] Raven Lord Icon
    { "Reins of the Raven Lord" },
}

Cryolysis2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
Cryolysis2:RegisterChatCommand("/cryolysis", "/cryo", options)
Cryolysis2:RegisterDB("Cryolysis2Config", "Cryolysis2ConfigPC")
Cryolysis2:RegisterDefaults("profile", {

    Language = GetLocale(),
	ProvisionContainer = 4,                 -- Where food/water is stored.  furthest bag on the left
	ProvisionSort = true,                   -- Sort food and drink
	ProvisionDestroy = false,               -- Destroy Excess food and dirnk
	ShowSpellTimers = false,				-- Show spell timers
	HPLimit = 85,							-- Eat food when under this % of health
	MPLimit = 100,							-- Drink water when under this % of mana
	MainButton = 1,							-- Main button function.  1 = None, 2 = eat/drink, 3 = Evocation, 4 = polymorph, 5 = mana gem, 6 = blink
	OuterSphere = 3,						-- Outside sphere display.  1 = None, 2 = HP, 3 = Mana, 4 = Mana Gem CD, 5 = Evocation CD
	InnerSphere = 5,                        -- Inside sphere display.  1 = none, 2 = food/drink, 3 = drink/food, 4 = Curr Health, 5 = Health %, 6 = Curr Mana, 7 = Mana %, 8 = Mana Gem CD, 9 = Evocation CD
    ShowFoodCountText = true,				-- Show food count on button
	ShowWaterCountText = true,				-- Show drink count on button
	ShowPowderCountText = false,            -- Show arcane powder count on button
	ShowFeatherCountText = true,            -- Show light feather count on button
	ShowManaGemCooldownText = false,        -- Show Mana gem cooldown on button
	ShowEvocationCooldownText = false,      -- Show evocation cooldown on button
	LockSphere = false,                     -- Lock everything in place
    LockButtons = true,                     -- Lock the buttons to the main sphere. (Keep them around the outside of the sphere.)
	CryolysisToolTip = true,                -- Show tooltips
	LeftSpellLeft,          -- Left spell button left click function: User can type in spell name
    LeftSpellMid,
    LeftSpellRight,
	RightSpellLeft,           -- Right spell button left click function: User can type in spell name 
    RightSpellMid,
    RightSpellRight,
    BottomSpellLeft,
    BottomSpellMid,
    BottomSpellRight,
	ChatMsg = true,
	ChatType = true,
	ShowCount = true,
	PolySounds = true,                      -- Play sounds for polymorph
	SM = false,                             -- Short messages
	PolyWarn = true,                        -- Warn when polymorph is about to break
	PolyBreak = true,                       -- Warn when polymorph breaks
	PolyWarnTime = 7,                       -- How long before polymorph breaks the user is warned.
	PolyMessage = true,                     -- Print messages when polymorphing
	TeleAndPortMessage = false,             -- Print messages when portaling
	MountMessage = false,                   -- Print messages when mounting up
	Restock	= true,				            -- Ask me if I want to restock.  If false, don't restock at all
	RestockIgnoreConfirm = true,	        -- Don't bother asking, just restock
	RestockTeleport = 10,			        -- Restock to 10 Rune of Teleportation
	RestockPortals = 10,			        -- Restock to 10 Rune of Portals
	RestockPowder = 20,				        -- Restock to 20 Arcane Powder
    Angle = 180,
    
    -- Variables for storing the Main Sphere's position
--~     MSPoint = "CENTER",
--~     MSRelTo = UIParent,
--~     MSRelPoint = "CENTER",
--~     MSx = 0,
--~     MSy = 0,

    -- Variables for scaling buttons and sphere and calculating constricted positions
    -- Radius and Scale are used for scaling.  Pos is used for switching button order.
    MainSphereRadius = 29,
    MainSphereScale = 1,
    DefaultButtonRadius = 15,
    FoodButtonScale = 1,
    FoodButtonPos = 1,
    WaterButtonScale = 1,
    WaterButtonPos = 19,
    ManaGemButtonScale = 1,
    ManaGemButtonPos = 17,
    LeftCustomButtonScale = 1,
    LeftCustomButtonPos = 15,
    EvocationButtonScale = 1,
    EvocationButtonPos = 13,
    RightCustomButtonScale = 1,
    RightCustomButtonPos = 11,
    SpellButtonScale = 1,
    SpellButtonPos = 9,
    MountButtonScale = 1,
    MountButtonPos = 7,
    TeleAndPortButtonScale = 1,
    TeleAndPortButtonPos = 5,
    BottomCustomButtonScale = 1,
    BottomCustomButtonPos = 3,
    
    MSHide = false,
    FoodHide = false,
    WaterHide = false,
    MountHide = false,
    BuffHide = false,
    PortHide = false,
    LeftHide = false,
    RightHide = false,
    BottomHide = false,
    GemHide = false,
    EvocHide = false,
    
} )

Cryolysis2Private = {

    -- Variables for tracking the status of the mage
    TradeWindowOpen = false,

    -- Variables for tracking whether the mage has certain spells/items
    HighestWaterId = 0,
    HighestWaterOnHand = 0,
	HighestWaterCount = 0,
	WaterName = {   
                "Conjured Water",
                "Conjured Fresh Water",
                "Conjured Purified Water",
                "Conjured Spring Water",
                "Conjured Mineral Water",
                "Conjured Sparkling Water",
                "Conjured Crystal Water",
                "Conjured Mountain Spring Water",
                "Conjured Glacier Water"
                }, -- Item IDs for Mage conjured water, in order from lowest amount of mana restored to highest.  ORDER DOES MATTER
                
    WaterTex = {
                "INV_Drink_06",
				"INV_Drink_07",
				"INV_Drink_Milk_02",
				"INV_Drink_10",
				"INV_Drink_09",
				"INV_Drink_11",
				"INV_Drink_18",
				"INV_Drink_Waterskin_11",
				"INV_Drink_16"
               },   -- Textures for the water button

	HighestFoodId = 0,
    HighestFoodOnHand = 0,
	HighestFoodCount = 0,
	FoodName = {
                "Conjured Muffin",
                "Conjured Bread",
                "Conjured Rye",
                "Conjured Pumpernickel",
                "Conjured Sourdough",
                "Conjured Sweet Roll",
                "Conjured Cinnamon Roll",
                "Conjured Croissant"
               }, -- Item IDs for Mage conjured bread, in order from lowest amount of health restored to highest.  ORDER DOES MATTER
                
    FoodTex = {
				"INV_Misc_Food_10",
				"INV_Misc_Food_09",
				"INV_Misc_Food_12",
				"INV_Misc_Food_08",
				"INV_Misc_Food_11",
				"INV_Misc_Food_33",
				"INV_Misc_Food_73CinnamonRoll",
				"INV_Misc_Food_32",
              },    -- Textures for the food button
    HighestManaGemId = 0,
    HighestManaGemOnHand = 0,
    ManaGemCooldownBool = false,
    ManaGemTex = {
                "INV_Misc_Gem_Emerald_01",
                "INV_Misc_Gem_Emerald_02",
                "INV_Misc_Gem_Opal_01",
                "INV_Misc_Gem_Ruby_01",
                "INV_Misc_Gem_Stone_01"
                },
    ManaGemName = {
                "Mana Agate",
                "Mana Jade",
                "Mana Citrine",
                "Mana Ruby",
                "Mana Emerald"
                },
                
	HasManaGem = { false, false, false, false, false },  -- Whether the user has a manastone conjured.  Ordered lowest to highest.
    
    HasEvocation = false,               -- Has the mage learned Evocation yet?
    HasArcaneIntellect = false,         -- Has the mage learned AI yet?
    HasFrostArmor = false,              -- Has the mage learned Frost Armor yet?
    HasIceArmor = false,                -- Has the mage learned Ice Armor yet?
    HasMoltenArmor = false,             -- Has the mage learned Molten Armor yet?
    HasMageArmor = false,               -- Has the mage learned Mage Armor yet?
    HasFireWard = false,                -- Has the mage learned Fire Ward yet?
    HasFrostWard = false,               -- Has the mage learned Frost Ward yet?
    HasSlowFall = false,                -- Has the mage learned Slow Fall yet?
    HasDetectMagic = false,             -- Has the mage learned Detect Magic yet?
    HasRemoveCurse = false,             -- Has the mage learned Remove Lesser Curse yet?
    HasManaShield = false,              -- Has the mage learned Mana Shield yet?
    HasIceBarrier = false,              -- Has the mage learned Ice Barrier? (talent based)
    HasDampenMagic = false,             -- Has the mage learned Dampen Magic yet?
    HasAmplifyMagic = false,            -- Has the mage learned Amplify Magic yet?
    HasArcaneBrilliance = false,        -- Has the mage learned Arcane Brilliance yet?
    
    -- Spell Menu Info: 1 = Armor, 2 = AI/AB, 3 = Amplify/Dampen, 4 = Ice Barrier/Mana Shield, 5 = Fire/Frost Ward, 6 = Detect Magic, 7 = Remove Lesser Curse, 8 = Slowfall
    HasSpellMenuBuff = {false, false, false, false, false, false, false, false},
    BuffMenu = {"Cryo2BuffMenu1", "Cryo2BuffMenu2", "Cryo2BuffMenu3", "Cryo2BuffMenu4", "Cryo2BuffMenu5", "Cryo2BuffMenu6", "Cryo2BuffMenu7", "Cryo2BuffMenu8"},
    BuffMenuVisible = false,
    
    -- Teleports/Portals: 1 = IF/Org, 2 = SW/UC, 3 = Darn/TB, 4 = Exodar/Silvermoon, 5 = Shat
    HasPortMenuPort = { false, false, false, false, false },
    HasPortMenuTeleport = { false, false, false, false, false },
    PortMenu = {"Cryo2PortMenu1", "Cryo2PortMenu2", "Cryo2PortMenu3", "Cryo2PortMenu4", "Cryo2PortMenu5"},
    PortMenuVisible = false,

	-- Cooldown variables
	EvocationCooldown = 0,
	EvocationCooldownText = "",
	ManastoneCooldown = 0,
	ManastoneCooldownText = "",
	ColdsnapCooldown = 0,
	ColdsnapCooldownText = "",
	IceblockCooldown = 0,
	IceblockCooldownText = "",
    
    -- Mount variables
    GroundMountString = "",
    GroundMountIcon = nil,
    HasGroundMount = false,
    GroundMountTex = {
        "INV_Misc_Horn_01",
        "Ability_Mount_MountainRam",
        "Ability_Mount_Raptor",
        "Ability_Mount_JungleTiger",
        "Ability_Mount_Undeadhorse",
        "Ability_Mount_MechaStrider",
        "Ability_Mount_RidingHorse",
        "Ability_Mount_Kodo_03",
        "Ability_Mount_NightmareHorse",
        "Ability_Mount_Kodo_01",
        "Ability_Mount_Kodo_02",
        "Ability_Mount_WhiteDireWolf",
        "Ability_Mount_BlackDireWolf",
        "Ability_Mount_BlackPanther",
        "Ability_Mount_WhiteTiger",
        "Ability_Mount_PinkTiger",
        "INV_Misc_QirajiCrystal_05",
        "INV_Misc_Foot_Centaur",
        "Ability_Mount_RidingElekk",
        "Ability_Mount_RidingElekk_Grey",
        "Ability_Mount_RidingElekk_Purple",
        "Ability_Mount_RidingElekkElite_Blue",
        "Ability_Mount_RidingElekkElite_Green",
        "Ability_Mount_RidingElekkElite_Purple",
        "Ability_Mount_CockatriceMount_Black",
        "Ability_Mount_CockatriceMount_Blue",
        "Ability_Mount_CockatriceMount_Purple",
        "Ability_Mount_CockatriceMount",
        "Ability_Mount_CockatriceMountElite_Green",
        "Ability_Mount_CockatriceMountElite",
        "Ability_Mount_CockatriceMountElite_Purple",
        "Ability_Mount_Dreadsteed",
    },
    
    FlyingMountString = "",
    FlyingMountIcon = nil,
    HasFlyingMount = false,
    FlyingMountTex = {
        "Ability_Mount_Gryphon_01",
        "Ability_Mount_Wyvern_01",
        "Ability_Mount_NetherdrakePurple",
        "Ability_Mount_NetherDrakeElite",
        "Ability_Hunter_Pet_NetherRay",
        "INV-Mount_Raven_54",
    }
    
};

function Cryolysis2:OnInitialize()

end

-- These events were pulled from the original Cryolysis, not sure if we'll need all of them.
function Cryolysis2:OnEnable()
    self:RegisterEvent("BAG_UPDATE")
    self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
    self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS")
    self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
    self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER")
    self:RegisterEvent("CHAT_MSG_SPELL_BREAK_AURA")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("MERCHANT_SHOW")
    self:RegisterEvent("MERCHANT_CLOSED")
    self:RegisterEvent("UNIT_SPELLCAST_FAILED")
    self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
    self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    self:RegisterEvent("UNIT_SPELLCAST_SENT")
    self:RegisterEvent("LEARNED_SPELL_IN_TAB")
    self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:RegisterEvent("TRADE_REQUEST_CANCEL")
    self:RegisterEvent("TRADE_SHOW")
    self:RegisterEvent("TRADE_CLOSED")
    self:RegisterEvent("VARIABLES_LOADED")
    self:RegisterEvent("PLAYER_LOGIN")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("UNIT_MANA")
    self:RegisterEvent("UNIT_HEALTH")
    self:RegisterEvent("Cryo2_UpdateEvocationCooldown")
    self:RegisterEvent("Cryo2_UpdateWardCooldown")
    self:RegisterEvent("Cryo2_UpdateManaGemCooldown")
end

function Cryolysis2:OnDisable()

end

---------------------------------------------------------
--Start of the GUI setup and handler functions
---------------------------------------------------------
function Cryolysis2:SetupSphereAndButtons()
    
    -- Main Sphere
    Cryo2MainSphere:SetAttribute("unit", "player")
    Cryo2MainSphere:Show()
    
    -- Evocation Button
    Cryo2EvocButtonIcon:SetTexture("Interface\\Icons\\Spell_Nature_Purge")
    Cryo2EvocButton:SetAttribute("unit", "player")
    Cryo2EvocButton:SetAttribute("type*", "spell")
    Cryo2EvocButton:SetAttribute("spell*", "Evocation")
    Cryo2EvocButton:Enable()
    Cryo2EvocButton:Show()
    
    -- Water Button
    Cryo2WaterButtonIcon:SetTexture("Interface\\Icons\\"..Cryolysis2Private.WaterTex[Cryolysis2Private.HighestWaterId])
    Cryo2WaterButton:SetAttribute("unit", "player")
    Cryo2WaterButton:SetAttribute("type2", "spell")
    Cryo2WaterButton:SetAttribute("spell2", "Conjure Water")
    Cryo2WaterButton:SetAttribute("type3", "Trade")
    Cryo2WaterButton.Trade = function() self:TradeWater() end
    Cryo2WaterButton:Enable()
    Cryo2WaterButton:Show()
    
    -- Food Button
    Cryo2FoodButtonIcon:SetTexture("Interface\\Icons\\"..Cryolysis2Private.FoodTex[Cryolysis2Private.HighestFoodId])
    Cryo2FoodButton:SetAttribute("unit", "player")
    Cryo2FoodButton:SetAttribute("type2", "spell")
    Cryo2FoodButton:SetAttribute("spell2", "Conjure Food")
    Cryo2FoodButton:SetAttribute("type3", "Trade")
    Cryo2FoodButton.Trade = function() self:TradeFood() end
    Cryo2FoodButton:Enable()
    Cryo2FoodButton:Show()
    
    -- Right Custom Button
    Cryo2RightCustomButton:SetAttribute("unit", "player")
    Cryo2RightCustomButton:SetAttribute("type1", "spell")
    Cryo2RightCustomButton:SetAttribute("type2", "spell")
    Cryo2RightCustomButton:SetAttribute("type3", "spell")
    Cryo2RightCustomButton:Enable()
    Cryo2RightCustomButton:Show()
    
    -- Left Custom Button
    Cryo2LeftCustomButton:SetAttribute("unit", "player")
    Cryo2LeftCustomButton:SetAttribute("type1", "spell")
    Cryo2LeftCustomButton:SetAttribute("type2", "spell")
    Cryo2LeftCustomButton:SetAttribute("type3", "spell")
    Cryo2LeftCustomButton:Enable()
    Cryo2LeftCustomButton:Show()
    
    -- Spell Menu Button
    Cryo2BuffMenuButtonIcon:SetTexture("Interface\\Icons\\INV_Staff_13")
    
    -- Mount Button
    Cryo2MountButton:Enable()
    Cryo2MountButton:Show()
    
    -- Teleport/Portal Menu Button
    Cryo2TeleAndPortMenuButtonIcon:SetTexture("Interface\\Icons\\Spell_Nature_AstralRecalGroup")
    
    -- Bottom Custom Button
    Cryo2BottomCustomButton:SetAttribute("unit", "player")
    Cryo2BottomCustomButton:SetAttribute("type1", "spell")
    Cryo2BottomCustomButton:SetAttribute("type2", "spell")
    Cryo2BottomCustomButton:SetAttribute("type3", "spell")
    Cryo2BottomCustomButton:Enable()
    Cryo2BottomCustomButton:Show()
    
end

function Cryolysis2:UpdateSphereAttributes()

    local MB = self.db.profile.MainButton
    
    if MB == 1 then
        Cryo2MainSphere:SetAttribute("type1", "macro")
        Cryo2MainSphere:SetAttribute("macrotext1", "")
    end
    if MB == 2 then
        self:EatAndDrinkUpdate()
    end
    if MB == 3 then
        Cryo2MainSphere:SetAttribute("type1", "spell")
        Cryo2MainSphere:SetAttribute("spell1", "Evocation")
    end
    if MB == 4 then
        Cryo2MainSphere:SetAttribute("type1", "spell")
        Cryo2MainSphere:SetAttribute("spell1", "Polymorph")
    end
    if MB == 5 then
        self:UseManaGemUpdate()
    end
    if MB == 6 then
        Cryo2MainSphere:SetAttribute("type1", "spell")
        Cryo2MainSphere:SetAttribute("spell1", "Blink")
    end

end

function Cryolysis2:UpdateAllButtonPositions()

    -- variables to make things easier to understand
    local CryoAngle = self.db.profile.Angle
    local MSSize = self.db.profile.MainSphereRadius * self.db.profile.MainSphereScale
    local FoodSize, FoodPos = self.db.profile.DefaultButtonRadius * self.db.profile.FoodButtonScale, self.db.profile.FoodButtonPos
    local WaterSize, WaterPos = self.db.profile.DefaultButtonRadius * self.db.profile.WaterButtonScale, self.db.profile.WaterButtonPos
    local ManaGemSize, ManaGemPos = self.db.profile.DefaultButtonRadius * self.db.profile.ManaGemButtonScale, self.db.profile.ManaGemButtonPos
    local LeftSize, LeftPos = self.db.profile.DefaultButtonRadius * self.db.profile.LeftCustomButtonScale, self.db.profile.LeftCustomButtonPos
    local EvocSize, EvocPos = self.db.profile.DefaultButtonRadius * self.db.profile.EvocationButtonScale, self.db.profile.EvocationButtonPos
    local RightSize, RightPos = self.db.profile.DefaultButtonRadius * self.db.profile.RightCustomButtonScale, self.db.profile.RightCustomButtonPos
    local SpellSize, SpellPos = self.db.profile.DefaultButtonRadius * self.db.profile.SpellButtonScale, self.db.profile.SpellButtonPos
    local MountSize, MountPos = self.db.profile.DefaultButtonRadius * self.db.profile.MountButtonScale, self.db.profile.MountButtonPos
    local TeleAndPortSize, TeleAndPortPos = self.db.profile.DefaultButtonRadius * self.db.profile.TeleAndPortButtonScale, self.db.profile.TeleAndPortButtonPos
    local BottomSize, BottomPos = self.db.profile.DefaultButtonRadius * self.db.profile.BottomCustomButtonScale, self.db.profile.BottomCustomButtonPos
    
    local tempcalc
    
    if self.db.profile.LockButtons == true then
        -- Food Button
        tempcalc = MSSize + FoodSize
        Cryo2FoodButton:ClearAllPoints()
        Cryo2FoodButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(FoodPos*18))), tempcalc*(sin(CryoAngle+(FoodPos*18))))
        
        -- Water Button
        tempcalc = MSSize+WaterSize
        Cryo2WaterButton:ClearAllPoints()
        Cryo2WaterButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(WaterPos*18))), tempcalc*(sin(CryoAngle+(WaterPos*18))))
        
        -- Mana Gem
        tempcalc = MSSize+ManaGemSize
        Cryo2ManaGemButton:ClearAllPoints()
        Cryo2ManaGemButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(ManaGemPos*18))), tempcalc*(sin(CryoAngle+(ManaGemPos*18))))
        
        -- Left Custom
        tempcalc = MSSize+LeftSize
        Cryo2LeftCustomButton:ClearAllPoints()
        Cryo2LeftCustomButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(LeftPos*18))), tempcalc*(sin(CryoAngle+(LeftPos*18))))
        
        -- Evocation Button
        tempcalc = MSSize+EvocSize
        Cryo2EvocButton:ClearAllPoints()
        Cryo2EvocButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(EvocPos*18))), tempcalc*(sin(CryoAngle+(EvocPos*18))))
        
        -- Right Custom
        tempcalc = MSSize+RightSize
        Cryo2RightCustomButton:ClearAllPoints()
        Cryo2RightCustomButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(RightPos*18))), tempcalc*(sin(CryoAngle+(RightPos*18))))
        
        -- Spell Menu Button
        tempcalc = MSSize+SpellSize
        Cryo2BuffMenuButton:ClearAllPoints()
        Cryo2BuffMenuButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(SpellPos*18))), tempcalc*(sin(CryoAngle+(SpellPos*18))))
        
        -- Mount Button
        tempcalc = MSSize+MountSize
        Cryo2MountButton:ClearAllPoints()
        Cryo2MountButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(MountPos*18))), tempcalc*(sin(CryoAngle+(MountPos*18))))
        
        -- Portal Button
        tempcalc = MSSize+TeleAndPortSize
        Cryo2TeleAndPortMenuButton:ClearAllPoints()
        Cryo2TeleAndPortMenuButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(TeleAndPortPos*18))), tempcalc*(sin(CryoAngle+(TeleAndPortPos*18))))
        
        tempcalc = MSSize+BottomSize
        Cryo2BottomCustomButton:ClearAllPoints()
        Cryo2BottomCustomButton:SetPoint("CENTER", "Cryo2MainSphere", "CENTER", tempcalc*(cos(CryoAngle+(BottomPos*18))), tempcalc*(sin(CryoAngle+(BottomPos*18))))
    end
        
end

function Cryolysis2:UpdateAllButtonSizes()

    Cryo2MainSphere:SetScale(self.db.profile.MainSphereScale)
    Cryo2FoodButton:SetScale(self.db.profile.FoodButtonScale)
    --Cryo2MainSphere:SetWidth(self.db.profile.MainSphereRadius * self.db.profile.MainSphereScale)
    --Cryo2FoodButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.FoodButtonScale)
    --Cryo2WaterButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.WaterButtonScale)
    --Cryo2MountButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.MountButtonScale)
    --Cryo2ManaGemButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.ManaGemButtonScale)
    --Cryo2LeftCustomButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.LeftButtonScale)
    --Cryo2RightCustomButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.RightButtonScale)
    --Cryo2EvocButton:SetWidth(self.db.profile.DefaultButtonRadius * self.db.profile.EvocationButtonScale)
    
    --Cryo2MainSphere:SetHeight(self.db.profile.MainSphereRadius * self.db.profile.MainSphereScale)
    --Cryo2FoodButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.FoodButtonScale)
    --Cryo2WaterButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.WaterButtonScale)
    --Cryo2MountButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.MountButtonScale)
    --Cryo2ManaGemButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.ManaGemButtonScale)
    --Cryo2LeftCustomButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.LeftButtonScale)
    --Cryo2RightCustomButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.RightButtonScale)
    --Cryo2EvocButton:SetHeight(self.db.profile.DefaultButtonRadius * self.db.profile.EvocationButtonScale)

end

function Cryolysis2:UpdateOuterSphere()
    
    local OS = self.db.profile.OuterSphere
    
    -- Update outer-sphere graphic based on the chosen attribute
    if OS == 1 then  -- Show nothing
        Cryo2MainSphere:SetNormalTexture("Interface\\Addons\\Cryolysis2\\UI\\Shard.tga")
    end
    if OS == 2 then  -- Show health
        local ChosenAttrib = (UnitHealth("player")/UnitHealthMax("player"))*100  -- Get's the player's health %
        ChosenAttrib = floor(ChosenAttrib/6.25)                                 -- Divide the health % by 6.25 and round down to see how many notches the fill
        Cryo2MainSphere:SetNormalTexture("Interface\\Addons\\Cryolysis2\\UI\\Shard"..ChosenAttrib..".tga")
    end
    if OS == 3 then  -- Show mana
        local ChosenAttrib = (UnitMana("player")/UnitManaMax("player"))*100  -- Get's the player's mana %
        ChosenAttrib = floor(ChosenAttrib/6.25)                             -- Divide the mana % by 6.25 and round down to see how many notches the fill
        Cryo2MainSphere:SetNormalTexture("Interface\\Addons\\Cryolysis2\\UI\\Shard"..ChosenAttrib..".tga")
    end
    if OS == 4 then  -- Show mana gem cooldown
    
        -- All cooldown tracking is handled by using custom events the reschedule themselves until the spell is ready to use again.
        -- This allows us to avoid constant polling of information.
    
    end
    if OS == 5 then  -- Show evocation cooldown
        
        -- All cooldown tracking is handled by using custom events the reschedule themselves until the spell is ready to use again.
        -- This allows us to avoid constant polling of information.
        
    end
end

function Cryolysis2:UpdateInnerSphere()
    
    local IS = self.db.profile.InnerSphere
    
    -- Update inner-sphere text based on the chosen attribute
    -- 1 = none, 2 = food/drink, 3 = drink/food, 4 = Curr Health, 5 = Health %, 6 = Curr Mana, 7 = Mana %, 8 = Mana Gem CD, 9 = Evocation CD
    if IS == 1 then
        Cryo2MainSphereText:SetText("")
    end
    if IS == 2 then
        Cryo2MainSphereText:SetTextColor(1, 1, 1, 1)
        Cryo2MainSphereText:SetText((self:CountFood()).."/"..(self:CountWater()))
    end
    if IS == 3 then
        Cryo2MainSphereText:SetTextColor(1, 1, 1, 1)
        Cryo2MainSphereText:SetText((self:CountWater()).."/"..(self:CountFood()))
    end
    if IS == 4 then
        local temphealth = UnitHealth("player")/UnitHealthMax("player")
        if temphealth >=0 and temphealth <=.5 then
            Cryo2MainSphereText:SetTextColor(1, temphealth*2, 0, 1)
        else
            Cryo2MainSphereText:SetTextColor((1-temphealth)*2, 1, 0, 1)
        end
        Cryo2MainSphereText:SetText(UnitHealth("player"))
    end
    if IS == 5 then
        local temphealth = UnitHealth("player")/UnitHealthMax("player")
        if temphealth >=0 and temphealth <=.50 then
            Cryo2MainSphereText:SetTextColor(1, temphealth*2, 0, 1)
        else
            Cryo2MainSphereText:SetTextColor((1-temphealth)*2, 1, 0, 1)
        end
        temphealth = temphealth*100
        Cryo2MainSphereText:SetText(floor(temphealth).."%")
    end
    if IS == 6 then
        Cryo2MainSphereText:SetTextColor(1, 1, 1, 1)
        Cryo2MainSphereText:SetText(UnitMana("player"))
    end
    if IS == 7 then
        Cryo2MainSphereText:SetTextColor(1, 1, 1, 1)
        local tempmana = UnitMana("player")/UnitManaMax("player")
        tempmana = tempmana*100
        Cryo2MainSphereText:SetText(floor(tempmana).."%")
    end
    if IS == 8 then
        -- All cooldown tracking is handled by using custom events the reschedule themselves until the spell is ready to use again.
        -- This allows us to avoid constant polling of information.
        self:Cryo2_UpdateManaGemCooldown()
    end
    if IS == 9 then
        -- All cooldown tracking is handled by using custom events the reschedule themselves until the spell is ready to use again.
        -- This allows us to avoid constant polling of information.
        self:Cryo2_UpdateEvocationCooldown()
    end
    
end

function Cryolysis2:UpdateFoodButton()
    
    self:FoodOnHand()
    
    local foodamount = self:CountFood()
    
    if self.db.profile.ShowFoodCountText == true then
        if foodamount == 0 then
            Cryo2FoodButtonText:SetText("")
        else
            Cryo2FoodButtonText:SetText(foodamount)
        end
    end
    
    if foodamount == 0 then
        Cryo2FoodButtonIcon:SetDesaturated(1)
    elseif foodamount > 0 and InCombatLockdown() ~= 1 then
        Cryo2FoodButtonIcon:SetDesaturated(nil)
    end
    
    Cryo2FoodButton:SetAttribute("type1", "item")
    Cryo2FoodButton:SetAttribute("item1", Cryolysis2Private.FoodName[Cryolysis2Private.HighestFoodOnHand])
    
end

function Cryolysis2:UpdateWaterButton()

    self:WaterOnHand()

    local wateramount = self:CountWater()

    if self.db.profile.ShowWaterCountText == true then
        if wateramount == 0 then
            Cryo2WaterButtonText:SetText("")
        else
            Cryo2WaterButtonText:SetText(wateramount)
        end
    end
    
    if wateramount == 0 then
        Cryo2WaterButtonIcon:SetDesaturated(1)
    elseif wateramount > 0 and InCombatLockdown() ~= 1 then
        Cryo2WaterButtonIcon:SetDesaturated(nil)
    end

    Cryo2WaterButton:SetAttribute("type1", "item")
    Cryo2WaterButton:SetAttribute("item1", Cryolysis2Private.WaterName[Cryolysis2Private.HighestWaterOnHand])
    
end

function Cryolysis2:UpdateRightButton(click)
    
    local spellID, spellName
    
    if click == 1 then
        spellName = self.db.profile.RightSpellLeft
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                local RightTex = GetSpellTexture(spellID, BOOKTYPE_SPELL)
                Cryo2RightCustomButtonIcon:SetTexture(RightTex)
                Cryo2RightCustomButton:SetAttribute("spell1", spellName)
                self.db.profile.RightSpellLeft = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
    
    if click == 2 then
        spellName = self.db.profile.RightSpellRight
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                Cryo2RightCustomButton:SetAttribute("spell2", spellName)
                self.db.profile.RightSpellRight = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
    
    if click == 3 then
        spellName = self.db.profile.RightSpellMid
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                Cryo2RightCustomButton:SetAttribute("spell3", spellName)
                self.db.profile.RightSpellMid = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end

end

function Cryolysis2:UpdateLeftButton(click)

    local spellID, spellName
    
    if click == 1 then
        spellName = self.db.profile.LeftSpellLeft
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                local LeftTex = GetSpellTexture(spellID, BOOKTYPE_SPELL)
                Cryo2LeftCustomButtonIcon:SetTexture(LeftTex)
                Cryo2LeftCustomButton:SetAttribute("spell1", spellName)
                self.db.profile.LeftSpellLeft = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
    
    if click == 2 then
        spellName = self.db.profile.LeftSpellRight
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                Cryo2LeftCustomButton:SetAttribute("spell2", spellName)
                self.db.profile.LeftSpellRight = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
    
    if click == 3 then
        spellName = self.db.profile.LeftSpellMid
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                Cryo2LeftCustomButton:SetAttribute("spell3", spellName)
                self.db.profile.LeftSpellMid = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
end

function Cryolysis2:UpdateBottomButton(click)

    local spellID, spellName
    
    if click == 1 then
        spellName = self.db.profile.BottomSpellLeft
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                local BottomTex = GetSpellTexture(spellID, BOOKTYPE_SPELL)
                Cryo2BottomCustomButtonIcon:SetTexture(BottomTex)
                Cryo2BottomCustomButton:SetAttribute("spell1", spellName)
                self.db.profile.BottomSpellLeft = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
    
    if click == 2 then
        spellName = self.db.profile.BottomSpellRight
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                Cryo2BottomCustomButton:SetAttribute("spell2", spellName)
                self.db.profile.BottomSpellRight = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
    
    if click == 3 then
        spellName = self.db.profile.BottomSpellMid
        if spellName ~= nil then
            spellID = self:FindSpellID(""..spellName, BOOKTYPE_SPELL)
            if spellID > 0 then
                Cryo2BottomCustomButton:SetAttribute("spell3", spellName)
                self.db.profile.BottomSpellMid = spellName
            else
                self:Print("Invalid spell, please check your spelling and try again!")
            end
        end
    end
end

function Cryolysis2:UpdateSpellMenu()

    Cryo2BuffMenuButton:SetAttribute("*childraise*", true)
    Cryo2BuffMenuButton:SetAttribute("*childstate*", "^click")
    Cryo2BuffMenu0:SetAttribute("statemap-anchor-click", "0-1")
    Cryo2BuffMenuButton:SetAttribute("anchorchild", Cryo2BuffMenu0)

    local lastBuffAdded = 0
    for i = 1, #(Cryolysis2Private.HasSpellMenuBuff), 1 do
        local CurrBuffMenu = getglobal(Cryolysis2Private.BuffMenu[i])
        if Cryolysis2Private.HasSpellMenuBuff[i] then
            if lastBuffAdded == 0 then
                CurrBuffMenu:ClearAllPoints()
                CurrBuffMenu:SetPoint("CENTER", "Cryo2BuffMenuButton", "CENTER", 33, 0)
                CurrBuffMenu:SetAttribute("showstates", "!0,*")
                Cryo2BuffMenu0:SetAttribute("addchild", CurrBuffMenu)
                lastBuffAdded = i
            else
                CurrBuffMenu:ClearAllPoints()
                CurrBuffMenu:SetPoint("CENTER", "Cryo2BuffMenu"..lastBuffAdded, "CENTER", 33, 0)
                CurrBuffMenu:SetAttribute("showstates", "!0,*")
                Cryo2BuffMenu0:SetAttribute("addchild", CurrBuffMenu)
                lastBuffAdded = i
            end
        end
    end
end

function Cryolysis2:UpdateBuffMenuAttributes()
    -- Armor Button
    Cryo2BuffMenu1:SetAttribute("unit", "player")
    if Cryolysis2Private.HasIceArmor == false and Cryolysis2Private.HasFrostArmor then
        Cryo2BuffMenu1:SetAttribute("type1", "spell")
        Cryo2BuffMenu1:SetAttribute("spell1", "Frost Armor")
    elseif Cryolysis2Private.HasIceArmor and Cryolysis2Private.HasFrostArmor then
        Cryo2BuffMenu1:SetAttribute("type1", "spell")
        Cryo2BuffMenu1:SetAttribute("spell1", "Ice Armor")
    end
    Cryo2BuffMenu1Icon:SetTexture("Interface\\Icons\\Spell_Frost_FrostArmor02")
    
    if Cryolysis2Private.HasMageArmor then
        Cryo2BuffMenu1:SetAttribute("type2", "spell")
        Cryo2BuffMenu1:SetAttribute("spell2", "Mage Armor")
    end
    
    if Cryolysis2Private.HasMoltenArmor then
        Cryo2BuffMenu1:SetAttribute("type3", "spell")
        Cryo2BuffMenu1:SetAttribute("spell3", "Molten Armor")
    end
    
    -- AI/AB
    Cryo2BuffMenu2Icon:SetTexture("Interface\\Icons\\Spell_Holy_MagicalSentry")
    if Cryolysis2Private.HasArcaneIntellect then
        Cryo2BuffMenu2:SetAttribute("type1", "spell")
        Cryo2BuffMenu2:SetAttribute("spell1", "Arcane Intellect")
    end
    if Cryolysis2Private.HasArcaneBrilliance then
        Cryo2BuffMenu2:SetAttribute("type2", "spell")
        Cryo2BuffMenu2:SetAttribute("spell2", "Arcane Brilliance")
    end
    
    -- Amplify/Dampen
    Cryo2BuffMenu3Icon:SetTexture("Interface\\Icons\\Spell_Nature_AbolishMagic")
    if Cryolysis2Private.HasDampenMagic then
        Cryo2BuffMenu3:SetAttribute("type1", "spell")
        Cryo2BuffMenu3:SetAttribute("spell1", "Dampen Magic")
    end
    
    if Cryolysis2Private.HasAmplifyMagic then
        Cryo2BuffMenu3:SetAttribute("type2", "spell")
        Cryo2BuffMenu3:SetAttribute("spell2", "Amplify Magic")
    end
    
    -- Ice Barrier/Mana Shield
    if Cryolysis2Private.HasIceBarrier then
        Cryo2BuffMenu4:SetAttribute("type1", "spell")
        Cryo2BuffMenu4:SetAttribute("spell1", "Ice Barrier")
        Cryo2BuffMenu4Icon:SetTexture("Interface\\Icons\\Spell_Ice_Lament")
    else
        Cryo2BuffMenu4:SetAttribute("type1", "spell")
        Cryo2BuffMenu4:SetAttribute("spell1", "Mana Shield")
        Cryo2BuffMenu4Icon:SetTexture("Interface\\Icons\\Spell_Shadow_DetectLesserInvisibility")
    end
    
    if Cryolysis2Private.HasManaShield then
        Cryo2BuffMenu4:SetAttribute("type2", "spell")
        Cryo2BuffMenu4:SetAttribute("spell2", "Mana Shield")
    end
    
    -- Fire/Frost Ward
    Cryo2BuffMenu5Icon:SetTexture("Interface\\Icons\\Spell_Fire_FireArmor")
    if Cryolysis2Private.HasFireWard then
        Cryo2BuffMenu5:SetAttribute("type1", "spell")
        Cryo2BuffMenu5:SetAttribute("spell1", "Fire Ward")
    end
    
    if Cryolysis2Private.HasFrostWard then
        Cryo2BuffMenu5:SetAttribute("type2", "spell")
        Cryo2BuffMenu5:SetAttribute("spell2", "Frost Ward")
    end
    
    -- Detect Magic
    Cryo2BuffMenu6Icon:SetTexture("Interface\\Icons\\Spell_Holy_Dizzy")
    if Cryolysis2Private.HasDetectMagic then
        Cryo2BuffMenu6:SetAttribute("type*", "spell")
        Cryo2BuffMenu6:SetAttribute("spell*", "Detect Magic")
    end
    
    -- Remove Lesser Curse
    Cryo2BuffMenu7Icon:SetTexture("Interface\\Icons\\Spell_Nature_RemoveCurse")
    if Cryolysis2Private.HasRemoveCurse then
        Cryo2BuffMenu7:SetAttribute("type*", "spell")
        Cryo2BuffMenu7:SetAttribute("spell*", "Remove Lesser Curse")
    end
    
    -- Slow Fall
    Cryo2BuffMenu8Icon:SetTexture("Interface\\Icons\\Spell_Magic_FeatherFall")
    if Cryolysis2Private.HasSlowFall then
        Cryo2BuffMenu8:SetAttribute("type*", "spell")
        Cryo2BuffMenu8:SetAttribute("spell*", "Slow Fall")
    end
end

function Cryolysis2:UpdateTeleAndPortButton()

-- Cryo2PortMenu0 inherits SecureStateHeaderTemplate
-- Cryo2TeleAndPortMenuButton inherits SecureAnchorButtonTemplate
-- Cryolysis2Private.HasPortMenuTeleport tracks which teleport spells the user has

    Cryo2TeleAndPortMenuButton:SetAttribute("*childraise*", true)
    Cryo2TeleAndPortMenuButton:SetAttribute("*childstate*", "^click")
    Cryo2PortMenu0:SetAttribute("statemap-anchor-click", "0-1")
    Cryo2TeleAndPortMenuButton:SetAttribute("anchorchild", Cryo2PortMenu0)
    
    local lastPortAdded = 0
    for i = 1, #(Cryolysis2Private.HasPortMenuTeleport), 1 do
        local CurrPortMenu = getglobal(Cryolysis2Private.PortMenu[i])
        if Cryolysis2Private.HasPortMenuTeleport[i] then   
            if lastPortAdded == 0 then
                CurrPortMenu:ClearAllPoints()
                CurrPortMenu:SetPoint("CENTER", "Cryo2TeleAndPortMenuButton", "CENTER", 33, 0)
                CurrPortMenu:SetAttribute("showstates", "!0,*")
                Cryo2PortMenu0:SetAttribute("addchild", CurrPortMenu)
                lastPortAdded = i
            else
                CurrPortMenu:ClearAllPoints()
                CurrPortMenu:SetPoint("CENTER", "Cryo2PortMenu"..lastPortAdded, "CENTER", 33, 0)
                CurrPortMenu:SetAttribute("showstates", "!0,*")
                Cryo2PortMenu0:SetAttribute("addchild", CurrPortMenu)
                lastPortAdded = i
            end
        end
    end
end

function Cryolysis2:UpdatePortMenuAttributes()

    local englishFaction, localizedFaction = UnitFactionGroup("player")
    
    if englishFaction == "Alliance" then
        if Cryolysis2Private.HasPortMenuTeleport[1] then
            Cryo2PortMenu1Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportIronForge")
            Cryo2PortMenu1:SetAttribute("type1", "spell")
            Cryo2PortMenu1:SetAttribute("spell1", "Teleport: Ironforge")
            if Cryolysis2Private.HasPortMenuPort[1] then
                Cryo2PortMenu1:SetAttribute("type2", "spell")
                Cryo2PortMenu1:SetAttribute("spell2", "Portal: Ironforge")
            end
        end
        if Cryolysis2Private.HasPortMenuTeleport[2] then
            Cryo2PortMenu2Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportStormWind")
            Cryo2PortMenu2:SetAttribute("type1", "spell")
            Cryo2PortMenu2:SetAttribute("spell1", "Teleport: Stormwind")
            if Cryolysis2Private.HasPortMenuPort[2] then
                Cryo2PortMenu2:SetAttribute("type2", "spell")
                Cryo2PortMenu2:SetAttribute("spell2", "Portal: Stormwind")
            end
        end
        if Cryolysis2Private.HasPortMenuTeleport[3] then
            Cryo2PortMenu3Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportDarnassus")
            Cryo2PortMenu3:SetAttribute("type1", "spell")
            Cryo2PortMenu3:SetAttribute("spell1", "Teleport: Darnassus")
            if Cryolysis2Private.HasPortMenuPort[3] then
                Cryo2PortMenu3:SetAttribute("type2", "spell")
                Cryo2PortMenu3:SetAttribute("spell2", "Portal: Darnassus")
            end
        end
        if Cryolysis2Private.HasPortMenuTeleport[4] then
            Cryo2PortMenu4Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportExodar")
            Cryo2PortMenu4:SetAttribute("type1", "spell")
            Cryo2PortMenu4:SetAttribute("spell1", "Teleport: Exodar")
            if Cryolysis2Private.HasPortMenuPort[4] then
                Cryo2PortMenu4:SetAttribute("type2", "spell")
                Cryo2PortMenu4:SetAttribute("spell2", "Portal: Exodar")
            end
        end
    end
    if englishFaction == "Horde" then
        if Cryolysis2Private.HasPortMenuTeleport[1] then
            Cryo2PortMenu1Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportOrgrimmar")
            Cryo2PortMenu1:SetAttribute("type1", "spell")
            Cryo2PortMenu1:SetAttribute("spell1", "Teleport: Orgrimmar")
            if Cryolysis2Private.HasPortMenuPort[1] then
                Cryo2PortMenu1:SetAttribute("type2", "spell")
                Cryo2PortMenu1:SetAttribute("spell2", "Portal: Orgrimmar")
            end
        end
        if Cryolysis2Private.HasPortMenuTeleport[2] then
            Cryo2PortMenu2Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportUnderCity")
            Cryo2PortMenu2:SetAttribute("type1", "spell")
            Cryo2PortMenu2:SetAttribute("spell1", "Teleport: Undercity")
            if Cryolysis2Private.HasPortMenuPort[2] then
                Cryo2PortMenu2:SetAttribute("type2", "spell")
                Cryo2PortMenu2:SetAttribute("spell2", "Portal: Undercity")
            end
        end
        if Cryolysis2Private.HasPortMenuTeleport[3] then
            Cryo2PortMenu3Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportThunderBluff")
            Cryo2PortMenu3:SetAttribute("type1", "spell")
            Cryo2PortMenu3:SetAttribute("spell1", "Teleport: Thunder Bluff")
            if Cryolysis2Private.HasPortMenuPort[3] then
                Cryo2PortMenu3:SetAttribute("type2", "spell")
                Cryo2PortMenu3:SetAttribute("spell2", "Portal: Thunder Bluff")
            end
        end
        if Cryolysis2Private.HasPortMenuTeleport[4] then
            Cryo2PortMenu4Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportSilvermoon")
            Cryo2PortMenu4:SetAttribute("type1", "spell")
            Cryo2PortMenu4:SetAttribute("spell1", "Teleport: Silvermoon")
            if Cryolysis2Private.HasPortMenuPort[4] then
                Cryo2PortMenu4:SetAttribute("type2", "spell")
                Cryo2PortMenu4:SetAttribute("spell2", "Portal: Silvermoon")
            end
        end
    end
    
    -- Both factions get this one!
    if Cryolysis2Private.HasPortMenuTeleport[5] then
        Cryo2PortMenu5Icon:SetTexture("Interface\\Icons\\Spell_Arcane_TeleportShattrath")
        Cryo2PortMenu5:SetAttribute("type1", "spell")
        Cryo2PortMenu5:SetAttribute("spell1", "Teleport: Shattrath")
        if Cryolysis2Private.HasPortMenuPort[5] then
            Cryo2PortMenu5:SetAttribute("type2", "spell")
            Cryo2PortMenu5:SetAttribute("spell2", "Portal: Shattrath")
        end
    end
end
        

function Cryolysis2:UpdateMountButton()

    if not Cryolysis2Private.HasGroundMount and not Cryolysis2Private.HasFlyingMount then  -- No mount
        Cryo2MountButtonIcon:SetTexture("Interface\\Icons\\INV_Misc_Rune_01")
        Cryo2MountButton:SetAttribute("type*", "item")			
        Cryo2MountButton:SetAttribute("item*", "Hearthstone")
    elseif Cryolysis2Private.HasGroundMount and not Cryolysis2Private.HasFlyingMount then  -- Ground Mount Only
        Cryo2MountButtonIcon:SetTexture("Interface\\Icons\\"..Cryolysis2Private.GroundMountTex[Cryolysis2Private.GroundMountIcon])
        Cryo2MountButton:SetAttribute("type1", "item")
        Cryo2MountButton:SetAttribute("item1", Cryolysis2Private.GroundMountString)
        Cryo2MountButton:SetAttribute("type2", "item")			
        Cryo2MountButton:SetAttribute("item2", "Hearthstone")
        Cryo2MountButton:SetAttribute("type3", "item")			
        Cryo2MountButton:SetAttribute("item3", "Hearthstone")
    elseif Cryolysis2Private.HasGroundMount and Cryolysis2Private.HasFlyingMount then
        Cryo2MountButtonIcon:SetTexture("Interface\\Icons\\"..Cryolysis2Private.FlyingMountTex[Cryolysis2Private.FlyingMountIcon])
        Cryo2MountButton:SetAttribute("type1", "item")
        Cryo2MountButton:SetAttribute("item1", Cryolysis2Private.FlyingMountString)
        Cryo2MountButton:SetAttribute("type2", "item")			
        Cryo2MountButton:SetAttribute("item2", Cryolysis2Private.GroundMountString)
        Cryo2MountButton:SetAttribute("type3", "item")			
        Cryo2MountButton:SetAttribute("item3", "Hearthstone")
    end

end

function Cryolysis2:UpdateManaGemButton()

    local MGFind
    for i = 1, #(Cryolysis2Private.ManaGemName), 1 do
        MGFind = self:BagFindCount(Cryolysis2Private.ManaGemName[i])
        if MGFind > 0 then
            Cryolysis2Private.HasManaGem[i] = true
            Cryolysis2Private.HighestManaGemOnHand = i
        else
            Cryolysis2Private.HasManaGem[i] = false
            Cryo2ManaGemButton:SetAttribute("type2", "spell")
            Cryo2ManaGemButton:SetAttribute("spell2", "Conjure "..Cryolysis2Private.ManaGemName[i])
        end
    end
    
    for i = 1, #(Cryolysis2Private.ManaGemName), 1 do
        if Cryolysis2Private.HasManaGem[i] == true then
            Cryo2ManaGemButtonIcon:SetTexture("Interface\\Icons\\"..Cryolysis2Private.ManaGemTex[i])
            Cryo2ManaGemButton:SetAttribute("type1", "item")
            Cryo2ManaGemButton:SetAttribute("item1", Cryolysis2Private.ManaGemName[i])
            --Cryo2ManaGemButtonIcon:SetDesaturated(nil)
        end
    end
    
    if not Cryolysis2Private.HasManaGem[1] and not Cryolysis2Private.HasManaGem[2] and not Cryolysis2Private.HasManaGem[3] and not Cryolysis2Private.HasManaGem[4] and not Cryolysis2Private.HasManaGem[5] then
        Cryo2ManaGemButtonIcon:SetTexture("Interface\\Icons\\"..Cryolysis2Private.ManaGemTex[Cryolysis2Private.HighestManaGemId])
        Cryo2ManaGemButtonIcon:SetDesaturated(1)
    end
    
    if Cryolysis2Private.ManaGemCooldownBool == false then
        self:ScheduleEvent("Cryo2_UpdateManaGemCooldown", 0)
    end

end

function Cryolysis2:UpdateHideShow()

    if Cryolysis2.db.profile.MSHide then
        Cryo2MainSphere:Hide()
    else
        Cryo2MainSphere:Show()
    end
    
    if Cryolysis2.db.profile.FoodHide then
        Cryo2FoodButton:Hide()
    else
        Cryo2FoodButton:Show()
    end

    if Cryolysis2.db.profile.WaterHide then
        Cryo2WaterButton:Hide()
    else
        Cryo2WaterButton:Show()
    end
    
    if Cryolysis2.db.profile.GemHide then
        Cryo2ManaGemButton:Hide()
    else
        Cryo2ManaGemButton:Show()
    end
    
    if Cryolysis2.db.profile.EvocHide then
        Cryo2EvocButton:Hide()
    else
        Cryo2EvocButton:Show()
    end
    
    if Cryolysis2.db.profile.PortHide then
        Cryo2TeleAndPortMenuButton:Hide()
    else
        Cryo2TeleAndPortMenuButton:Show()
    end
    
    if Cryolysis2.db.profile.BuffHide then
        Cryo2BuffMenuButton:Hide()
    else
        Cryo2BuffMenuButton:Show()
    end
    
    if Cryolysis2.db.profile.MountHide then
        Cryo2MountButton:Hide()
    else
        Cryo2MountButton:Show()
    end
    
    if Cryolysis2.db.profile.LeftHide then
        Cryo2LeftCustomButton:Hide()
    else
        Cryo2LeftCustomButton:Show()
    end
    
    if Cryolysis2.db.profile.RightHide then
        Cryo2RightCustomButton:Hide()
    else
        Cryo2RightCustomButton:Show()
    end
    
    if Cryolysis2.db.profile.BottomHide then
        Cryo2BottomCustomButton:Hide()
    else
        Cryo2BottomCustomButton:Show()
    end
    
end
---------------------------------------------------------
--End of the GUI setup and handler functions
---------------------------------------------------------

---------------------------------------------------------
--Start of the event handler functions
---------------------------------------------------------

function Cryolysis2:BAG_UPDATE()

    self:UpdateWaterButton()
    self:UpdateFoodButton()
    self:UpdateManaGemButton()
    self:EatAndDrinkUpdate()
    self:UseManaGemUpdate()

end

function Cryolysis2:CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE()

end

function Cryolysis2:CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS()

end

function Cryolysis2:CHAT_MSG_SPELL_AURA_GONE_SELF()

end

function Cryolysis2:CHAT_MSG_SPELL_AURA_GONE_OTHER()

end

function Cryolysis2:CHAT_MSG_SPELL_BREAK_AURA()

end

function Cryolysis2:PLAYER_REGEN_DISABLED()

    Cryo2FoodButtonIcon:SetDesaturated(1)
    Cryo2WaterButtonIcon:SetDesaturated(1)
    Cryo2MountButtonIcon:SetDesaturated(1)

end

function Cryolysis2:PLAYER_REGEN_ENABLED()

    self:UpdateFoodButton()
    self:UpdateWaterButton()

end

function Cryolysis2:MERCHANT_SHOW()

end

function Cryolysis2:MERCHANT_CLOSED()

end

function Cryolysis2:UNIT_SPELLCAST_FAILED()

end

function Cryolysis2:UNIT_SPELLCAST_INTERRUPTED()

end

function Cryolysis2:UNIT_SPELLCAST_SUCCEEDED(unit, name, rank)

    if name == "Evocation" then
        self:ScheduleEvent("Cryo2_UpdateEvocationCooldown", 0)
    end
    if name == "Fire Ward" then
        self:ScheduleEvent("Cryo2_UpdateWardCooldown", 0)
    end
    if name == "Frost Ward" then
        self:ScheduleEvent("Cryo2_UpdateWardCooldown", 0)
    end

end

function Cryolysis2:UNIT_SPELLCAST_SENT()

end

function Cryolysis2:LEARNED_SPELL_IN_TAB()
    self:DetectSpellsAndRanks()
    self:UpdateSpellMenu()
    self:UpdateBuffMenuAttributes()
    self:UpdateTeleAndPortButton()
    self:UpdatePortMenuAttributes()
end

function Cryolysis2:CHAT_MSG_SPELL_SELF_DAMAGE()

end

function Cryolysis2:PLAYER_TARGET_CHANGED()

end

function Cryolysis2:TRADE_REQUEST_CANCEL()
    Cryolysis2Private.TradeWindowOpen = false
end

function Cryolysis2:TRADE_SHOW()
    Cryolysis2Private.TradeWindowOpen = true
end

function Cryolysis2:TRADE_CLOSED()

    Cryolysis2Private.TradeWindowOpen = false
    self:UpdateFoodButton()
    self:UpdateWaterButton()

end

function Cryolysis2:VARIABLES_LOADED()

end

function Cryolysis2:PLAYER_LOGIN()
end

function Cryolysis2:PLAYER_ENTERING_WORLD()
    self:DetectSpellsAndRanks()
    self:SetupSphereAndButtons()
    self:UpdateAllButtonPositions()
    self:UpdateSphereAttributes()
    self:UpdateFoodButton()
    self:UpdateWaterButton()
    self:FindGroundMount()
    self:FindFlyingMount()
    self:UpdateMountButton()
    self:UpdateLeftButton(1)
    self:UpdateRightButton(1)
    self:UpdateBottomButton(1)
    self:UpdateLeftButton(2)
    self:UpdateRightButton(2)
    self:UpdateBottomButton(2)
    self:UpdateLeftButton(3)
    self:UpdateRightButton(3)
    self:UpdateBottomButton(3)
    self:UpdateManaGemButton()
    self:Cryo2_UpdateEvocationCooldown()
    self:Cryo2_UpdateManaGemCooldown()
    self:UpdateInnerSphere()
    self:UpdateOuterSphere()
    self:UpdateSpellMenu()
    self:UpdateBuffMenuAttributes()
    self:UpdateTeleAndPortButton()
    self:UpdatePortMenuAttributes()
    dewdrop:Register(Cryo2MainSphere, "children", options, 'cursorX', true, 'cursorY', true)
end

function Cryolysis2:UNIT_MANA(unit)
    if unit == "player" then
        if self.db.profile.InnerSphere == 2 or self.db.profile.InnerSphere == 3 or self.db.profile.InnerSphere == 6 or self.db.profile.InnerSphere == 7 then 
            self:UpdateInnerSphere()
        end
        if self.db.profile.OuterSphere == 3 then
            self:UpdateOuterSphere()
        end
    end
end

function Cryolysis2:UNIT_HEALTH(unit)
    if unit == "player" then
        if self.db.profile.InnerSphere == 2 or self.db.profile.InnerSphere == 3 or self.db.profile.InnerSphere == 4 or self.db.profile.InnerSphere == 5 then
            self:UpdateInnerSphere()
        end
        if self.db.profile.OuterSphere == 2 then
            self:UpdateOuterSphere()
        end
    end
end

function Cryolysis2:Cryo2_UpdateEvocationCooldown()
    
    if self.db.profile.ShowEvocationCooldownText or self.db.profile.InnerSphere == 9 then
        local start, duration, enabled = GetSpellCooldown("Evocation")
        if duration == 0 then       -- Spell is not on cooldown
            if self.db.profile.ShowEvocationCooldownText then
                Cryo2EvocButtonText:SetText("")
            end
            if self.db.profile.InnerSphere == 9 then
                Cryo2MainSphereText:SetText("Ready")
            end
            Cryo2EvocButtonIcon:SetDesaturated(nil)
            self:CancelScheduledEvent("Cryo2_UpdateEvocationCooldown")
        else
            local endtime = start + duration
            local timeleft = endtime - GetTime()
            local minutes = floor(timeleft/60)
            local seconds = floor(math.fmod(timeleft, 60))
            Cryo2EvocButtonIcon:SetDesaturated(1)
            if self.db.profile.ShowEvocationCooldownText then
                if minutes > 0 then
                    if seconds < 10 then
                        Cryo2EvocButtonText:SetText(minutes..":0"..seconds)
                    else
                        Cryo2EvocButtonText:SetText(minutes..":"..seconds)
                    end
                else
                    Cryo2EvocButtonText:SetText(floor(timeleft))
                end
            else
                Cryo2EvocButtonText:SetText("")
            end
            if self.db.profile.InnerSphere == 9 then
                if minutes > 0 then
                    if seconds < 10 then
                        Cryo2MainSphereText:SetText(minutes..":0"..seconds)
                    else
                        Cryo2MainSphereText:SetText(minutes..":"..seconds)
                    end
                else
                    Cryo2MainSphereText:SetText(floor(timeleft))
                end
            end
            self:ScheduleEvent("Cryo2_UpdateEvocationCooldown", 1)
        end
    end
end

function Cryolysis2:Cryo2_UpdateWardCooldown()

    local start, duration, enabled = GetSpellCooldown("Fire Ward")
    if duration == 0 then       -- Spell is not on cooldown
        self:CancelScheduledEvent("Cryo2_UpdateWardCooldown")
    else
        self:ScheduleEvent("Cryo2_UpdateWardCooldown", 1)
    end

end

function Cryolysis2:Cryo2_UpdateManaGemCooldown()

-- When BAG_UPDATE is fired, check the mage's bags against stored values of mana gems on-hand.
-- If the state of any of the gems has changed, then schedule this event to repeat.
    if self.db.profile.ShowManaGemCooldownText or self.db.profile.InnerSphere == 8 then
        if Cryolysis2Private.HighestManaGemOnHand > 0 then
            local start, duration, enabled = GetItemCooldown(Cryolysis2Private.ManaGemName[Cryolysis2Private.HighestManaGemOnHand])
            if duration == 0 then       -- Spell is not on cooldown and a gem is conjured
                if self.db.profile.ShowManaGemCooldownText then
                    Cryo2ManaGemButtonText:SetText("")
                end
                if self.db.profile.InnerSphere == 8 then
                    Cryo2MainSphereText:SetText("Ready")
                end
                Cryo2ManaGemButtonIcon:SetDesaturated(nil)
                self:CancelScheduledEvent("Cryo2_UpdateManaGemCooldown")
                Cryolysis2Private.ManaGemCooldownBool = false
            else
                local endtime = start + duration
                local timeleft = endtime - GetTime()
                local minutes = floor(timeleft/60)
                local seconds = floor(math.fmod(timeleft, 60))
                Cryo2ManaGemButtonIcon:SetDesaturated(1)
                Cryolysis2Private.ManaGemCooldownBool = true
                if self.db.profile.ShowManaGemCooldownText then
                    if minutes > 0 then
                        if seconds < 10 then
                            Cryo2ManaGemButtonText:SetText(minutes..":0"..seconds)
                        else
                            Cryo2ManaGemButtonText:SetText(minutes..":"..seconds)
                        end
                    else
                        Cryo2ManaGemButtonText:SetText(floor(timeleft))
                    end
                else
                    Cryo2ManaGemButtonText:SetText("")
                end
                if self.db.profile.InnerSphere == 8 then
                    if minutes > 0 then
                        if seconds < 10 then
                            Cryo2MainSphereText:SetText(minutes..":0"..seconds)
                        else
                            Cryo2MainSphereText:SetText(minutes..":"..seconds)
                        end
                    else
                        Cryo2MainSphereText:SetText(floor(timeleft))
                    end
                end
                self:ScheduleEvent("Cryo2_UpdateManaGemCooldown", 1)
            end
        else -- no gems conjured
            if self.db.profile.ShowManaGemCooldownText then
                Cryo2ManaGemButtonText:SetText("")
                Cryo2ManaGemButtonIcon:SetDesaturated(1)
            end
            if self.db.profile.InnerSphere == 8 then
                Cryo2MainSphereText:SetText("Unavailable")
            end
        end
    end
end

---------------------------------------------------------
-- End of event handlers
---------------------------------------------------------


---------------------------------------------------------
-- Start of functions for handling spell/action requests
---------------------------------------------------------

function Cryolysis2:Polymorph()

end

function Cryolysis2:EatAndDrinkUpdate()

    if self.db.profile.MainButton == 2 then
        self:WaterOnHand()
        self:FoodOnHand()
        Cryo2MainSphere:SetAttribute("type1", "macro")
        if Cryolysis2Private.HighestWaterOnHand > 0 and Cryolysis2Private.HighestFoodOnHand > 0 then
            Cryo2MainSphere:SetAttribute("macrotext1", "/use "..Cryolysis2Private.WaterName[Cryolysis2Private.HighestWaterOnHand].."\n/use "..Cryolysis2Private.FoodName[Cryolysis2Private.HighestFoodOnHand])
        end
        if Cryolysis2Private.HighestWaterOnHand > 0 and Cryolysis2Private.HighestFoodOnHand == 0 then
            Cryo2MainSphere:SetAttribute("macrotext1", "/use "..Cryolysis2Private.WaterName[Cryolysis2Private.HighestWaterOnHand])
        end
        if Cryolysis2Private.HighestWaterOnHand == 0 and Cryolysis2Private.HighestFoodOnHand > 0 then
            Cryo2MainSphere:SetAttribute("macrotext1", "/use "..Cryolysis2Private.FoodName[Cryolysis2Private.HighestFoodOnHand])
        end
        if Cryolysis2Private.HighestWaterOnHand == 0 and Cryolysis2Private.HighestFoodOnHand == 0 then
            Cryo2MainSphere:SetAttribute("macrotext1", "")
        end
    end
end

function Cryolysis2:CountWater()

    for i = #(Cryolysis2Private.WaterName), 1, -1 do
        if self:BagFindCount(Cryolysis2Private.WaterName[i]) > 0 then
            return self:BagFindCount(Cryolysis2Private.WaterName[i])
        end
    end
    return 0
    
end

function Cryolysis2:WaterOnHand()

    Cryolysis2Private.HighestWaterOnHand = 0

    for i= 1, #(Cryolysis2Private.WaterName), 1 do
        if self:BagFindCount(Cryolysis2Private.WaterName[i]) > 0 then
            Cryolysis2Private.HighestWaterOnHand = i
        end
    end

end

function Cryolysis2:CountFood()

    for i = #(Cryolysis2Private.FoodName), 1, -1 do
        if self:BagFindCount(Cryolysis2Private.FoodName[i]) > 0 then
            return self:BagFindCount(Cryolysis2Private.FoodName[i])
        end
    end
    return 0
    
end

function Cryolysis2:FoodOnHand()
    
    Cryolysis2Private.HighestFoodOnHand = 0
    
    for i= 1, #(Cryolysis2Private.FoodName), 1 do
        if self:BagFindCount(Cryolysis2Private.FoodName[i]) > 0 then
            Cryolysis2Private.HighestFoodOnHand = i
        end
    end
    
end

function Cryolysis2:FindGroundMount()

    for icon = 1, #(GroundMounts), 1 do
		for i = 1, #(GroundMounts[icon]), 1 do
            if self:BagFindCount(GroundMounts[icon][i]) > 0 then
                Cryolysis2Private.GroundMountString = GroundMounts[icon][i]
                Cryolysis2Private.GroundMountIcon = icon
                Cryolysis2Private.HasGroundMount = true
                return;
            end
        end
    end
   
end

function Cryolysis2:FindFlyingMount()

    for icon = 1, #(FlyingMounts), 1 do
		for i = 1, #(FlyingMounts[icon]), 1 do
            if self:BagFindCount(FlyingMounts[icon][i]) > 0 then
                Cryolysis2Private.FlyingMountString = FlyingMounts[icon][i]
                Cryolysis2Private.FlyingMountIcon = icon
                Cryolysis2Private.HasFlyingMount = true
                return;
            end
        end
    end
   
end

function Cryolysis2:TradeFood()

    if (UnitExists("target") and UnitIsPlayer("target") and UnitIsFriend("player", "target") and UnitName("target") ~= UnitName("player")) then
        
        self:FoodOnHand()
        
        if Cryolysis2Private.TradeWindowOpen == false then
            InitiateTrade("target")
        end
        
        if Cryolysis2Private.HighestFoodOnHand > 0 then
            local bag, slot = self:BagFindSlot(Cryolysis2Private.FoodName[Cryolysis2Private.HighestFoodOnHand])
            PickupContainerItem(bag, slot)
            DropItemOnUnit("target")
        else
            self:Print("No food found for trading, conjure some then try again!")
        end
        
    end

end

function Cryolysis2:TradeWater()
    if (UnitExists("target") and UnitIsPlayer("target") and UnitIsFriend("player", "target") and UnitName("target") ~= UnitName("player")) then
        
        self:WaterOnHand()
        
        if Cryolysis2Private.TradeWindowOpen == false then
            InitiateTrade("target")
        end
        
        if Cryolysis2Private.HighestWaterOnHand > 0 then
            local bag, slot = self:BagFindSlot(Cryolysis2Private.WaterName[Cryolysis2Private.HighestWaterOnHand])
            PickupContainerItem(bag, slot)
            DropItemOnUnit("target")
        else
            self:Print("No water found for trading, conjure some then try again!")
        end
    end
end

function Cryolysis2:UseManaGemUpdate()

    if self.db.profile.MainButton == 5 then
        Cryo2MainSphere:SetAttribute("type1", "macro")
        
        for i = 1, #(Cryolysis2Private.HasManaGem), 1 do
            if Cryolysis2Private.HasManaGem[i] then
                Cryo2MainSphere:SetAttribute("macrotext1", "/use "..Cryolysis2Private.ManaGemName[i])
            else
                Cryo2MainSphere:SetAttribute("macrotext1", "")
            end
        end
    end
    
end

-- Function used to detect what spells the mage has.
-- For food and water it also detects the highest rank the mage has.
-- This is so we can add in casting of a certain rank of the spell according to the target's level.
-- This function should be called once initially while loading in, and then anytime the mage learns a new spell.
-- THESE STRINGS NEED TO BE LOCALIZED!
function Cryolysis2:DetectSpellsAndRanks()
    local i = 1
    local spellRankText, spellRankNumber
    while true do
        local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
        if not spellName then
            do break end
        end
        
        -- These ones are straight-forward, if we find the spell, set the corresponding value to true
        if string.find(spellName, "Arcane Intellect") then
            Cryolysis2Private.HasArcaneIntellect = true
            Cryolysis2Private.HasSpellMenuBuff[2] = true
        end
        if string.find(spellName, "Arcane Brilliance") then
            Cryolysis2Private.HasArcaneBrilliance = true
            Cryolysis2Private.HasSpellMenuBuff[2] = true
        end
        if string.find(spellName, "Frost Armor") then
            Cryolysis2Private.HasFrostArmor = true
            Cryolysis2Private.HasSpellMenuBuff[1] = true
        end
        if string.find(spellName, "Ice Armor") then
            Cryolysis2Private.HasIceArmor = true
            Cryolysis2Private.HasSpellMenuBuff[1] = true
        end
        if string.find(spellName, "Molten Armor") then
            Cryolysis2Private.HasMoltenArmor = true
            Cryolysis2Private.HasSpellMenuBuff[1] = true
        end
        if string.find(spellName, "Mage Armor") then
            Cryolysis2Private.HasMageArmor = true
            Cryolysis2Private.HasSpellMenuBuff[1] = true
        end
        if string.find(spellName, "Remove Lesser Curse") then
            Cryolysis2Private.HasRemoveCurse = true
            Cryolysis2Private.HasSpellMenuBuff[7] = true
        end
        if string.find(spellName, "Slow Fall") then
            Cryolysis2Private.HasSlowFall = true
            Cryolysis2Private.HasSpellMenuBuff[8] = true
        end
        if string.find(spellName, "Evocation") then
            Cryolysis2Private.HasEvocation = true
        end
        if string.find(spellName, "Mana Shield") then
            Cryolysis2Private.HasManaShield = true
            Cryolysis2Private.HasSpellMenuBuff[4] = true
        end
        if string.find(spellName, "Ice Barrier") then
            Cryolysis2Private.HasIceBarrier = true
            Cryolysis2Private.HasSpellMenuBuff[4] = true
        end
        if string.find(spellName, "Dampen Magic") then
            Cryolysis2Private.HasDampenMagic = true
            Cryolysis2Private.HasSpellMenuBuff[3] = true
        end
        if string.find(spellName, "Amplify Magic") then
            Cryolysis2Private.HasAmplifyMagic = true
            Cryolysis2Private.HasSpellMenuBuff[3] = true
        end
        if string.find(spellName, "Detect Magic") then
            Cryolysis2Private.HasDetectMagic = true
            Cryolysis2Private.HasSpellMenuBuff[6] = true
        end
        if string.find(spellName, "Fire Ward") then
            Cryolysis2Private.HasFireWard = true
            Cryolysis2Private.HasSpellMenuBuff[5] = true
        end
        if string.find(spellName, "Frost Ward") then
            Cryolysis2Private.HasFrostWard = true
            Cryolysis2Private.HasSpellMenuBuff[5] = true
        end
        
        -- Teleports and Portals
        if string.find(spellName, "Teleport: Ironforge") or string.find(spellName, "Teleport: Orgrimmar") then
            Cryolysis2Private.HasPortMenuTeleport[1] = true
        end
        if string.find(spellName, "Teleport: Stormwind") or string.find(spellName, "Teleport: Undercity") then
            Cryolysis2Private.HasPortMenuTeleport[2] = true
        end
        if string.find(spellName, "Teleport: Darnassus") or string.find(spellName, "Teleport: Thunder Bluff") then
            Cryolysis2Private.HasPortMenuTeleport[3] = true
        end
        if string.find(spellName, "Teleport: Exodar") or string.find (spellName, "Teleport: Silvermoon") then
            Cryolysis2Private.HasPortMenuTeleport[4] = true
        end
        if string.find(spellName, "Teleport: Shattrath") then
            Cryolysis2Private.HasPortMenuTeleport[5] = true
        end
        if string.find(spellName, "Portal: Ironforge") or string.find(spellName, "Portal: Orgrimmar") then
            Cryolysis2Private.HasPortMenuPort[1] = true
        end
        if string.find(spellName, "Portal: Stormwind") or string.find(spellName, "Portal: Undercity") then
            Cryolysis2Private.HasPortMenuPort[2] = true
        end
        if string.find(spellName, "Portal: Darnassus") or string.find(spellName, "Portal: Thunder Bluff") then
            Cryolysis2Private.HasPortMenuPort[3] = true
        end
        if string.find(spellName, "Portal: Exodar") or string.find (spellName, "Portal: Silvermoon") then
            Cryolysis2Private.HasPortMenuPort[4] = true
        end
        if string.find(spellName, "Portal: Shattrath") then
            Cryolysis2Private.HasPortMenuPort[5] = true
        end
        
        -- These ones are slightly more complex.  If we find the spell, we need to check the rank against the currently known rank.
        -- If the rank we found is higher, then we change the highest know rank to the one we found, otherwise we do nothing.
        -- For the mana stones, we have to statically set the rank numbers to compare, since there's no rank information for them.
        spellRankText, spellRankNumber = strsplit(" ", spellRank)
        if tonumber(spellRankNumber) ~= nil then
            if string.find(spellName, "Conjure Water") then
                if tonumber(spellRankNumber) > Cryolysis2Private.HighestWaterId then
                    Cryolysis2Private.HighestWaterId = tonumber(spellRankNumber)
                end
            end
            if string.find(spellName, "Conjure Food") then
                if tonumber(spellRankNumber) > Cryolysis2Private.HighestFoodId then
                    Cryolysis2Private.HighestFoodId = tonumber(spellRankNumber)
                end
            end
        end
        if string.find(spellName, "Conjure Mana Agate") then
            if 1 > Cryolysis2Private.HighestManaGemId then
                Cryolysis2Private.HighestManaGemId = 1
            end
        end
        if string.find(spellName, "Conjure Mana Jade") then
            if 2 > Cryolysis2Private.HighestManaGemId then
                Cryolysis2Private.HighestManaGemId = 2
            end
        end
        if string.find(spellName, "Conjure Mana Citrine") then
            if 3 > Cryolysis2Private.HighestManaGemId then
                Cryolysis2Private.HighestManaGemId = 3
            end
        end
        if string.find(spellName, "Conjure Mana Ruby") then
            if 4 > Cryolysis2Private.HighestManaGemId then
                Cryolysis2Private.HighestManaGemId = 4
            end
        end
        if string.find(spellName, "Conjure Mana Emerald") then
            if 5 > Cryolysis2Private.HighestManaGemId then
                Cryolysis2Private.HighestManaGemId = 5
            end
        end
        i = i + 1
    end
end

function Cryolysis2:FindSpellID(findname)

    local i = 1
    if string.len(findname) > 0 then
        while true do
            local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
            if not spellName then
                do break end
            end
            
            if spellName == findname then
                return i
            end
            i = i + 1
        end
    end
    return 0

end

-- Searches the mage's bags and counts the item specified.
-- Returns the amount of the item specified.
-- item = a string
function Cryolysis2:BagFindCount(item)
    --local count = 0
    --local itemString
    --local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, invTexture, texture, itemCount, locked, quality, readable
    -- For each bag
--~     for bag = 0, 4, 1 do
--~         -- For each slot in the bag being checked
--~         for slot = 1, GetContainerNumSlots(bag), 1 do
--~             if (GetContainerItemLink(bag,slot)) then
--~                 itemString = ""..GetContainerItemLink(bag,slot)
--~                 while (itemString == nil) do                        -- itemString should never return nil, but it does due to lag sometimes.  We need to keep checking the bag slot until the item finishes creating.
--~                     self:Print("Infinite Loop!")
--~                     itemString = ""..GetContainerItemLink(bag,slot)
--~                 end
--~                 itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, invTexture = GetItemInfo(itemString)
--~                 if string.find(itemName, item) then
--~                     texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
--~                     count = count + itemCount
--~                 end
--~             end
--~         end
--~     end
    return GetItemCount(item)
end

-- Searches the mage's bags for the given item.
-- Returns the item's bag and slot location if found.
-- item = a string
function Cryolysis2:BagFindSlot(item)
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, invTexture, itemString
    for bag = 0, 4, 1 do
        for slot = 1, GetContainerNumSlots(bag), 1 do
            if (GetContainerItemLink(bag,slot)) then
            itemString = ""..GetContainerItemLink(bag,slot)
            itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, invTexture = GetItemInfo(itemString)
                if string.find(itemName, item) then
                    return bag, slot;
                end
            end
        end
    end
end
